/**
 * 
 */
package net.java.amateras.uml.sequence.editpart;

import java.beans.PropertyChangeEvent;

import net.java.amateras.uml.editpart.AbstractUMLEntityEditPart;
import net.java.amateras.uml.model.AbstractUMLEntityModel;
import net.java.amateras.uml.sequence.figure.ActorFigure;
import net.java.amateras.uml.sequence.figure.InstanceFigure;
import net.java.amateras.uml.sequence.model.ActivationModel;
import net.java.amateras.uml.sequence.model.ActorModel;
import net.java.amateras.uml.sequence.model.InstanceModel;
import net.java.amateras.uml.sequence.model.InteractionModel;
import net.java.amateras.uml.sequence.model.LifeLineModel;
import net.java.amateras.uml.sequence.model.MessageModel;
import net.java.amateras.uml.sequence.model.MessageOrderUtil;
import net.java.amateras.uml.sequence.model.SyncMessageModel;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ComponentEditPolicy;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.GroupRequest;
import org.eclipse.gef.requests.ReconnectRequest;

/**
 * @author Takahiro Shida.
 * 
 */
public class InstanceEditPart extends AbstractUMLEntityEditPart {

	/*
	 * ( Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		InstanceModel model = (InstanceModel) getModel();
		InstanceFigure figure = null;
		if (model instanceof ActorModel) {
			figure = new ActorFigure().build();
		} else {
			figure = new InstanceFigure().build();
		}
		if (model.getType() != null) {
			figure.setInstanceName(model.getName() + " : " + model.getType().getName());
		} else {
			figure.setInstanceName(model.getName());
		}
		return figure;
	}

	protected void createEditPolicies() {
		super.createEditPolicies();
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new ComponentEditPolicy() {
			protected Command createDeleteCommand(GroupRequest deleteRequest) {
				InstanceModel model = (InstanceModel) getModel();
				if (model.getModel().getChildren().isEmpty()) {
					DeleteCommand command = new DeleteCommand();
					command.setModel(model);
					return command;
				}
				return super.createDeleteCommand(deleteRequest);
			}
		});
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new InstanceNodeEditPolicy());
	}

	/*
	 * ( Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(AbstractUMLEntityModel.P_CONSTRAINT)) {
			refreshVisuals();
		} else if (evt.getPropertyName().equals(InstanceModel.P_NAME)
				|| evt.getPropertyName().equals(InstanceModel.P_TYPE)) {
			refreshName();
		} else if (evt.getPropertyName().equals(
				AbstractUMLEntityModel.P_SOURCE_CONNECTION)) {
			refreshSourceConnections();
		} else if (evt.getPropertyName().equals(
				AbstractUMLEntityModel.P_TARGET_CONNECTION)) {
			refreshTargetConnections();
		}

	}

	private InstanceModel getInstance() {
		return (InstanceModel) getModel();
	}

	private void refreshName() {
		InstanceModel model = (InstanceModel) getModel();
		String name = model.getName();
		if (model.getType() != null) {
			name += " : " + model.getType().getName();
		}
		InstanceFigure figure = (InstanceFigure) getFigure();
		figure.setInstanceName(name);
	}

	public ConnectionAnchor getTargetConnectionAnchor(
			ConnectionEditPart connection) {
		MessageModel model = (MessageModel) connection.getModel();
		if (model instanceof SyncMessageModel) {
			return new InstanceIncomingConnectionAnchor(getFigure(), model
					.getSource());
		}
		return null;
	}

	class InstanceNodeEditPolicy extends GraphicalNodeEditPolicy {

		protected Command getConnectionCompleteCommand(
				CreateConnectionRequest request) {
			CreateCompleteCommand command = new CreateCompleteCommand();
			ConnectionCommand locationCommand = (ConnectionCommand) request
					.getStartCommand();
			command.setPoint(locationCommand.getPoint());
			command.setModel(locationCommand.getModel());
			command.setSource(locationCommand.getSource());
			return locationCommand.chain(command);
		}

		protected Command getConnectionCreateCommand(
				CreateConnectionRequest request) {
			return null;
		}

		protected Command getReconnectTargetCommand(ReconnectRequest request) {
			return null;
		}

		protected Command getReconnectSourceCommand(ReconnectRequest request) {
			return null;
		}

	}

	private class CreateCompleteCommand extends ConnectionCommand {

		public boolean canExecute() {
			InstanceModel model = getInstance();
			return model.getModel().getChildren().isEmpty();
		}

		public void execute() {
			// RlNV͂.
			Rectangle rectangle = getInstance().getConstraint().getCopy();
			Point p = rectangle.getTopLeft();
			p.y = getPoint().y;
			rectangle.setLocation(p);
			getInstance().setConstraint(rectangle);
			getModel().setSource(getSource());
			getModel().setTarget(getInstance());
			getModel().attachSource();
			getModel().attachTarget();
			ActivationModel act = (ActivationModel) getSource();
			act.computeCaller();
			MessageOrderUtil.computeMessageOrders(getInstance().getRoot());
			super.execute();
		}

		public void undo() {
			super.undo();
			getModel().detachSource();
			getModel().detachTarget();
			Rectangle rectangle = getInstance().getConstraint().getCopy();
			Point p = rectangle.getTopLeft();
			p.y = InstanceModel.DEFAULT_LOCATION;
			rectangle.setLocation(p);
			getInstance().setConstraint(rectangle);
			MessageOrderUtil.computeMessageOrders(getInstance().getRoot());
		}
	}

	class DeleteCommand extends Command {
		InstanceModel model;

		LifeLineModel lineModel;

		InteractionModel interactionModel;

		public void setModel(InstanceModel model) {
			this.model = model;
			lineModel = model.getModel();
			interactionModel = model.getRoot();
		}

		public void execute() {
			super.execute();
			interactionModel.removeInstance(model);
			interactionModel.removeLifeLine(lineModel);
		}

		public void undo() {
			super.undo();
			interactionModel.addInstance(model);
		}
	}

}
